@@ -32,6 +32,7 @@ void ags_feed_audio_signal_init(AgsFeedAudioSignal *feed_audio_signal);
 void ags_feed_audio_signal_finalize(GObject *gobject);
 
 void ags_feed_audio_signal_run_pre(AgsRecall *recall);
+void ags_feed_audio_signal_run_post(AgsRecall *recall);
 
 /**
  * SECTION:ags_feed_audio_signal
@@ -94,6 +95,7 @@ ags_feed_audio_signal_class_init(AgsFeedAudioSignalClass *feed_audio_signal)
   recall = (AgsRecallClass *) feed_audio_signal;
 
   recall->run_pre = ags_feed_audio_signal_run_pre;
+  recall->run_post = ags_feed_audio_signal_run_post;
 }
 
 void
@@ -117,12 +119,13 @@ void
 ags_feed_audio_signal_run_pre(AgsRecall *recall)
 {
   AgsAudioSignal *template, *audio_signal;
+
   AgsFeedAudioSignal *feed_audio_signal;
 
   GObject *output_soundcard;
   
   GList *note_start, *note;
-  
+
   void (*parent_class_run_pre)(AgsRecall *recall);
 
   pthread_mutex_t *recall_mutex;
@@ -155,7 +158,7 @@ ags_feed_audio_signal_run_pre(AgsRecall *recall)
   g_object_get(audio_signal,
 	       "note", &note_start,
 	       NULL);
-
+  
   if(note_start != NULL){
     note = note_start;
 
@@ -164,7 +167,8 @@ ags_feed_audio_signal_run_pre(AgsRecall *recall)
 	AgsRecycling *recycling;
 
 	guint x0, x1;
-	guint samplerate;
+	guint buffer_size;
+	guint note_offset;
 	gdouble notation_delay;
 	guint frame_count;
 
@@ -177,7 +181,7 @@ ags_feed_audio_signal_run_pre(AgsRecall *recall)
 	
 	g_object_get(audio_signal,
 		     "recycling", &recycling,
-		     "samplerate", &samplerate,
+		     "buffer-size", &buffer_size,
 		     NULL);
       
 	if(recycling != NULL){
@@ -197,9 +201,10 @@ ags_feed_audio_signal_run_pre(AgsRecall *recall)
     
 	/* get notation delay */
 	notation_delay = ags_soundcard_get_absolute_delay(AGS_SOUNDCARD(output_soundcard));
+	note_offset = ags_soundcard_get_note_offset(AGS_SOUNDCARD(output_soundcard));
 
 	/* feed audio signal */
-	frame_count = (guint) (((gdouble) samplerate / notation_delay) * (gdouble) (x1 - x0 + 2));
+	frame_count = (guint) (((gdouble) buffer_size * notation_delay) * (gdouble) ((note_offset + 1) - x0));
 
 	ags_audio_signal_feed(audio_signal,
 			      template,
@@ -208,8 +213,6 @@ ags_feed_audio_signal_run_pre(AgsRecall *recall)
 
       note = note->next;
     }
-  }else{
-    ags_recall_done(recall);
   }
 
   /* unref */
@@ -221,6 +224,67 @@ ags_feed_audio_signal_run_pre(AgsRecall *recall)
 		   g_object_unref);
 }
 
+void
+ags_feed_audio_signal_run_post(AgsRecall *recall)
+{
+  AgsAudioSignal *audio_signal;
+
+  AgsFeedAudioSignal *feed_audio_signal;
+
+  GList *note_start, *note;
+
+  gboolean success;
+  
+  void (*parent_class_run_post)(AgsRecall *recall);
+
+  pthread_mutex_t *recall_mutex;
+  
+  feed_audio_signal = (AgsFeedAudioSignal *) recall;
+
+  /* get mutex */
+  pthread_mutex_lock(ags_recall_get_class_mutex());
+
+  recall_mutex = recall->obj_mutex;
+
+  parent_class_run_post = AGS_RECALL_CLASS(ags_feed_audio_signal_parent_class)->run_post;
+
+  pthread_mutex_unlock(ags_recall_get_class_mutex());
+
+  /* call parent */
+  parent_class_run_post(recall);
+  
+  g_object_get(feed_audio_signal,
+	       "source", &audio_signal,
+	       NULL);
+
+  g_object_get(audio_signal,
+	       "note", &note_start,
+	       NULL);
+
+  note = note_start;
+  success = FALSE;
+  
+  while(note != NULL){      
+    if(ags_note_test_flags(note->data, AGS_NOTE_FEED)){
+      success = TRUE;
+
+      break;
+    }
+
+    note = note->next;
+  }
+
+  if(!success){
+    ags_recall_done(recall);
+  }
+
+  /* unref */
+  g_object_unref(audio_signal);
+  
+  g_list_free_full(note_start,
+		   g_object_unref);
+}
+
 /**
  * ags_feed_audio_signal_new:
  * @source: the #AgsAudioSignal
